@@ -93,15 +93,15 @@
 #define PAL_FRAMERATE_NUMERATOR 25
 #define PAL_FRAMERATE_DENOMINATOR 1
 
-#define PAL_NORMAL_PAR_X        59
-#define PAL_NORMAL_PAR_Y        54
-#define PAL_WIDE_PAR_X          118
-#define PAL_WIDE_PAR_Y          81
+#define PAL_NORMAL_PAR_X        16
+#define PAL_NORMAL_PAR_Y        15
+#define PAL_WIDE_PAR_X          64
+#define PAL_WIDE_PAR_Y          45
 
-#define NTSC_NORMAL_PAR_X       10
-#define NTSC_NORMAL_PAR_Y       11
-#define NTSC_WIDE_PAR_X         40
-#define NTSC_WIDE_PAR_Y         33
+#define NTSC_NORMAL_PAR_X       8
+#define NTSC_NORMAL_PAR_Y       9
+#define NTSC_WIDE_PAR_X         32
+#define NTSC_WIDE_PAR_Y         27
 
 GST_DEBUG_CATEGORY_STATIC (dvdemux_debug);
 #define GST_CAT_DEFAULT dvdemux_debug
@@ -182,12 +182,9 @@ gst_dvdemux_class_init (GstDVDemuxClass * klass)
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_dvdemux_change_state);
   gstelement_class->send_event = GST_DEBUG_FUNCPTR (gst_dvdemux_send_event);
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sink_temp));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&video_src_temp));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&audio_src_temp));
+  gst_element_class_add_static_pad_template (gstelement_class, &sink_temp);
+  gst_element_class_add_static_pad_template (gstelement_class, &video_src_temp);
+  gst_element_class_add_static_pad_template (gstelement_class, &audio_src_temp);
 
   gst_element_class_set_static_metadata (gstelement_class,
       "DV system stream demuxer", "Codec/Demuxer",
@@ -258,7 +255,7 @@ gst_dvdemux_reset (GstDVDemux * dvdemux)
   dvdemux->frame_offset = 0;
   dvdemux->audio_offset = 0;
   dvdemux->video_offset = 0;
-  dvdemux->framecount = 0;
+  dvdemux->discont = TRUE;
   g_atomic_int_set (&dvdemux->found_header, 0);
   dvdemux->frame_len = -1;
   dvdemux->need_segment = FALSE;
@@ -271,6 +268,8 @@ gst_dvdemux_reset (GstDVDemux * dvdemux)
   dvdemux->wide = FALSE;
   gst_segment_init (&dvdemux->byte_segment, GST_FORMAT_BYTES);
   gst_segment_init (&dvdemux->time_segment, GST_FORMAT_TIME);
+  dvdemux->segment_seqnum = 0;
+  dvdemux->upstream_time_segment = FALSE;
   dvdemux->have_group_id = FALSE;
   dvdemux->group_id = G_MAXUINT;
 }
@@ -640,6 +639,57 @@ gst_dvdemux_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
       gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
       break;
     }
+    case GST_QUERY_SEEKING:
+    {
+      GstFormat fmt;
+      GstQuery *peerquery;
+      gboolean seekable;
+
+      gst_query_parse_seeking (query, &fmt, NULL, NULL, NULL);
+
+      /* We can only handle TIME seeks really */
+      if (fmt != GST_FORMAT_TIME) {
+        gst_query_set_seeking (query, fmt, FALSE, -1, -1);
+        break;
+      }
+
+      /* First ask upstream */
+      if (gst_pad_peer_query (dvdemux->sinkpad, query)) {
+        gst_query_parse_seeking (query, NULL, &seekable, NULL, NULL);
+        if (seekable) {
+          res = TRUE;
+          break;
+        }
+      }
+
+      res = TRUE;
+
+      peerquery = gst_query_new_seeking (GST_FORMAT_BYTES);
+      seekable = gst_pad_peer_query (dvdemux->sinkpad, peerquery);
+
+      if (seekable)
+        gst_query_parse_seeking (peerquery, NULL, &seekable, NULL, NULL);
+      gst_query_unref (peerquery);
+
+      if (seekable) {
+        peerquery = gst_query_new_duration (GST_FORMAT_TIME);
+        seekable = gst_dvdemux_src_query (pad, parent, peerquery);
+
+        if (seekable) {
+          gint64 duration;
+
+          gst_query_parse_duration (peerquery, NULL, &duration);
+          gst_query_set_seeking (query, GST_FORMAT_TIME, seekable, 0, duration);
+        } else {
+          gst_query_set_seeking (query, GST_FORMAT_TIME, FALSE, -1, -1);
+        }
+
+        gst_query_unref (peerquery);
+      } else {
+        gst_query_set_seeking (query, GST_FORMAT_TIME, FALSE, -1, -1);
+      }
+      break;
+    }
     default:
       res = gst_pad_query_default (pad, parent, query);
       break;
@@ -732,6 +782,7 @@ gst_dvdemux_handle_sink_event (GstPad * pad, GstObject * parent,
       GST_DEBUG ("cleared adapter");
       gst_segment_init (&dvdemux->byte_segment, GST_FORMAT_BYTES);
       gst_segment_init (&dvdemux->time_segment, GST_FORMAT_TIME);
+      dvdemux->discont = TRUE;
       res = gst_dvdemux_push_event (dvdemux, event);
       break;
     case GST_EVENT_SEGMENT:
@@ -743,11 +794,15 @@ gst_dvdemux_handle_sink_event (GstPad * pad, GstObject * parent,
         case GST_FORMAT_BYTES:
           gst_segment_copy_into (segment, &dvdemux->byte_segment);
           dvdemux->need_segment = TRUE;
+          dvdemux->segment_seqnum = gst_event_get_seqnum (event);
           gst_event_unref (event);
           break;
         case GST_FORMAT_TIME:
           gst_segment_copy_into (segment, &dvdemux->time_segment);
 
+          dvdemux->upstream_time_segment = TRUE;
+          dvdemux->segment_seqnum = gst_event_get_seqnum (event);
+
           /* and we can just forward this time event */
           res = gst_dvdemux_push_event (dvdemux, event);
           break;
@@ -912,6 +967,7 @@ gst_dvdemux_handle_push_seek (GstDVDemux * dvdemux, GstPad * pad,
     /* now this is the updated seek event on bytes */
     newevent = gst_event_new_seek (rate, GST_FORMAT_BYTES, flags,
         cur_type, start_position, stop_type, end_position);
+    gst_event_set_seqnum (newevent, gst_event_get_seqnum (event));
 
     res = gst_pad_push_event (dvdemux->sinkpad, newevent);
   }
@@ -919,6 +975,23 @@ done:
   return res;
 }
 
+static void
+gst_dvdemux_update_frame_offsets (GstDVDemux * dvdemux, GstClockTime timestamp)
+{
+  /* calculate current frame number */
+  gst_dvdemux_src_convert (dvdemux, dvdemux->videosrcpad,
+      dvdemux->time_segment.format, timestamp,
+      GST_FORMAT_DEFAULT, &dvdemux->video_offset);
+
+  /* calculate current audio number */
+  gst_dvdemux_src_convert (dvdemux, dvdemux->audiosrcpad,
+      dvdemux->time_segment.format, timestamp,
+      GST_FORMAT_DEFAULT, &dvdemux->audio_offset);
+
+  /* every DV frame corresponts with one video frame */
+  dvdemux->frame_offset = dvdemux->video_offset;
+}
+
 /* position ourselves to the configured segment, used in pull mode.
  * The input segment is in TIME format. We convert the time values
  * to bytes values into our byte_segment which we use to pull data from
@@ -955,18 +1028,9 @@ gst_dvdemux_do_seek (GstDVDemux * demux, GstSegment * segment)
       segment->format, segment->time, format,
       (gint64 *) & demux->byte_segment.time);
 
-  /* calculate current frame number */
-  format = GST_FORMAT_DEFAULT;
-  gst_dvdemux_src_convert (demux, demux->videosrcpad,
-      segment->format, segment->start, format, &demux->video_offset);
+  gst_dvdemux_update_frame_offsets (demux, segment->start);
 
-  /* calculate current audio number */
-  format = GST_FORMAT_DEFAULT;
-  gst_dvdemux_src_convert (demux, demux->audiosrcpad,
-      segment->format, segment->start, format, &demux->audio_offset);
-
-  /* every DV frame corresponts with one video frame */
-  demux->frame_offset = demux->video_offset;
+  demux->discont = TRUE;
 
 done:
   return res;
@@ -989,6 +1053,7 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
   gboolean flush;
   gboolean update;
   GstSegment seeksegment;
+  GstEvent *new_event;
 
   GST_DEBUG_OBJECT (demux, "doing seek");
 
@@ -1015,13 +1080,18 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
     flags = 0;
   }
 
+  demux->segment_seqnum = gst_event_get_seqnum (event);
+
   flush = flags & GST_SEEK_FLAG_FLUSH;
 
   /* send flush start */
-  if (flush)
-    gst_dvdemux_push_event (demux, gst_event_new_flush_start ());
-  else
+  if (flush) {
+    new_event = gst_event_new_flush_start ();
+    gst_event_set_seqnum (new_event, demux->segment_seqnum);
+    gst_dvdemux_push_event (demux, new_event);
+  } else {
     gst_pad_pause_task (demux->sinkpad);
+  }
 
   /* grab streaming lock, this should eventually be possible, either
    * because the task is paused or our streaming thread stopped
@@ -1049,7 +1119,9 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
   if (flush) {
     /* send flush stop, peer will accept data and events again. We
      * are not yet providing data as we still have the STREAM_LOCK. */
-    gst_dvdemux_push_event (demux, gst_event_new_flush_stop (TRUE));
+    new_event = gst_event_new_flush_stop (TRUE);
+    gst_event_set_seqnum (new_event, demux->segment_seqnum);
+    gst_dvdemux_push_event (demux, new_event);
   }
 
   /* if successfull seek, we update our real segment and push
@@ -1058,9 +1130,12 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
     memcpy (&demux->time_segment, &seeksegment, sizeof (GstSegment));
 
     if (demux->time_segment.flags & GST_SEEK_FLAG_SEGMENT) {
-      gst_element_post_message (GST_ELEMENT_CAST (demux),
-          gst_message_new_segment_start (GST_OBJECT_CAST (demux),
-              demux->time_segment.format, demux->time_segment.position));
+      GstMessage *message;
+
+      message = gst_message_new_segment_start (GST_OBJECT_CAST (demux),
+          demux->time_segment.format, demux->time_segment.position);
+      gst_message_set_seqnum (message, demux->segment_seqnum);
+      gst_element_post_message (GST_ELEMENT_CAST (demux), message);
     }
     if ((stop = demux->time_segment.stop) == -1)
       stop = demux->time_segment.duration;
@@ -1072,6 +1147,7 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
       gst_event_unref (demux->pending_segment);
 
     demux->pending_segment = gst_event_new_segment (&demux->time_segment);
+    gst_event_set_seqnum (demux->pending_segment, demux->segment_seqnum);
 
     demux->need_segment = FALSE;
   }
@@ -1125,10 +1201,9 @@ gst_dvdemux_send_event (GstElement * element, GstEvent * event)
       } else {
         GST_OBJECT_UNLOCK (dvdemux);
 
-        if (dvdemux->seek_handler) {
+        if (dvdemux->seek_handler)
           res = dvdemux->seek_handler (dvdemux, dvdemux->videosrcpad, event);
-          gst_event_unref (event);
-        }
+        gst_event_unref (event);
       }
       break;
     }
@@ -1245,7 +1320,7 @@ gst_dvdemux_demux_audio (GstDVDemux * dvdemux, GstBuffer * buffer,
     dvdemux->audio_offset += num_samples;
     GST_BUFFER_OFFSET_END (outbuf) = dvdemux->audio_offset;
 
-    if (dvdemux->new_media)
+    if (dvdemux->new_media || dvdemux->discont)
       GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
 
     ret = gst_pad_push (dvdemux->audiosrcpad, outbuf);
@@ -1325,7 +1400,7 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
   GST_BUFFER_OFFSET_END (outbuf) = dvdemux->video_offset + 1;
   GST_BUFFER_DURATION (outbuf) = duration;
 
-  if (dvdemux->new_media)
+  if (dvdemux->new_media || dvdemux->discont)
     GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
 
   GST_DEBUG ("pushing video %" GST_TIME_FORMAT,
@@ -1427,8 +1502,11 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
   GstSMPTETimeCode timecode;
   int frame_number;
 
-  if (G_UNLIKELY (dvdemux->need_segment)) {
+  if (dvdemux->need_segment) {
     GstFormat format;
+    GstEvent *event;
+
+    g_assert (!dvdemux->upstream_time_segment);
 
     /* convert to time and store as start/end_timestamp */
     format = GST_FORMAT_TIME;
@@ -1439,15 +1517,14 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
                 (gint64 *) & dvdemux->time_segment.stop)))
       goto segment_error;
 
+    dvdemux->time_segment.time = dvdemux->time_segment.start;
     dvdemux->time_segment.rate = dvdemux->byte_segment.rate;
-    dvdemux->time_segment.position = dvdemux->time_segment.start;
 
-    /* calculate current frame number */
-    format = GST_FORMAT_DEFAULT;
-    if (!(gst_dvdemux_src_convert (dvdemux, dvdemux->videosrcpad,
-                GST_FORMAT_TIME, dvdemux->time_segment.start,
-                format, &dvdemux->frame_offset)))
-      goto segment_error;
+    gst_dvdemux_sink_convert (dvdemux,
+        GST_FORMAT_BYTES, dvdemux->byte_segment.position,
+        GST_FORMAT_TIME, (gint64 *) & dvdemux->time_segment.position);
+
+    gst_dvdemux_update_frame_offsets (dvdemux, dvdemux->time_segment.position);
 
     GST_DEBUG_OBJECT (dvdemux, "sending segment start: %" GST_TIME_FORMAT
         ", stop: %" GST_TIME_FORMAT ", time: %" GST_TIME_FORMAT,
@@ -1455,8 +1532,10 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
         GST_TIME_ARGS (dvdemux->time_segment.stop),
         GST_TIME_ARGS (dvdemux->time_segment.start));
 
-    gst_dvdemux_push_event (dvdemux,
-        gst_event_new_segment (&dvdemux->time_segment));
+    event = gst_event_new_segment (&dvdemux->time_segment);
+    if (dvdemux->segment_seqnum)
+      gst_event_set_seqnum (event, dvdemux->segment_seqnum);
+    gst_dvdemux_push_event (dvdemux, event);
 
     dvdemux->need_segment = FALSE;
   }
@@ -1501,6 +1580,7 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
     goto done;
   }
 
+  dvdemux->discont = FALSE;
   dvdemux->time_segment.position = next_ts;
   dvdemux->frame_offset++;
 
@@ -1596,14 +1676,28 @@ gst_dvdemux_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
   /* a discontinuity in the stream, we need to get rid of
    * accumulated data in the adapter and assume a new frame
    * starts after the discontinuity */
-  if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT)))
+  if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT))) {
     gst_adapter_clear (dvdemux->adapter);
+    dvdemux->discont = TRUE;
+
+    /* Should recheck where we are */
+    if (!dvdemux->upstream_time_segment)
+      dvdemux->need_segment = TRUE;
+  }
 
   /* a timestamp always should be respected */
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
   if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
     dvdemux->time_segment.position = timestamp;
-    /* FIXME, adjust frame_offset and other counters */
+
+    if (dvdemux->discont)
+      gst_dvdemux_update_frame_offsets (dvdemux,
+          dvdemux->time_segment.position);
+  } else if (dvdemux->upstream_time_segment && dvdemux->discont) {
+    /* This will probably fail later to provide correct
+     * timestamps and/or durations but also should not happen */
+    GST_ERROR_OBJECT (dvdemux,
+        "Upstream provides TIME segment but no PTS after discont");
   }
 
   gst_adapter_push (dvdemux->adapter, buffer);
@@ -1746,20 +1840,30 @@ done:
   /* ERRORS */
 parse_header_error:
   {
+    GstEvent *event;
+
     GST_ELEMENT_ERROR (dvdemux, STREAM, DECODE,
         (NULL), ("Error parsing DV header"));
     gst_buffer_unref (buffer);
     gst_pad_pause_task (dvdemux->sinkpad);
-    gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
+    event = gst_event_new_eos ();
+    if (dvdemux->segment_seqnum)
+      gst_event_set_seqnum (event, dvdemux->segment_seqnum);
+    gst_dvdemux_push_event (dvdemux, event);
     goto done;
   }
 small_buffer:
   {
+    GstEvent *event;
+
     GST_ELEMENT_ERROR (dvdemux, STREAM, DECODE,
         (NULL), ("Error reading buffer"));
     gst_buffer_unref (buffer);
     gst_pad_pause_task (dvdemux->sinkpad);
-    gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
+    event = gst_event_new_eos ();
+    if (dvdemux->segment_seqnum)
+      gst_event_set_seqnum (event, dvdemux->segment_seqnum);
+    gst_dvdemux_push_event (dvdemux, event);
     goto done;
   }
 pause:
@@ -1777,20 +1881,35 @@ pause:
         dvdemux->time_segment.position = dvdemux->time_segment.start;
       /* perform EOS logic */
       if (dvdemux->time_segment.flags & GST_SEEK_FLAG_SEGMENT) {
-        gst_element_post_message (GST_ELEMENT (dvdemux),
-            gst_message_new_segment_done (GST_OBJECT_CAST (dvdemux),
-                dvdemux->time_segment.format, dvdemux->time_segment.position));
-        gst_dvdemux_push_event (dvdemux,
-            gst_event_new_segment_done (dvdemux->time_segment.format,
-                dvdemux->time_segment.position));
+        GstMessage *message;
+        GstEvent *event;
+
+        event = gst_event_new_segment_done (dvdemux->time_segment.format,
+            dvdemux->time_segment.position);
+        if (dvdemux->segment_seqnum)
+          gst_event_set_seqnum (event, dvdemux->segment_seqnum);
+
+        message = gst_message_new_segment_done (GST_OBJECT_CAST (dvdemux),
+            dvdemux->time_segment.format, dvdemux->time_segment.position);
+        if (dvdemux->segment_seqnum)
+          gst_message_set_seqnum (message, dvdemux->segment_seqnum);
+
+        gst_element_post_message (GST_ELEMENT (dvdemux), message);
+        gst_dvdemux_push_event (dvdemux, event);
       } else {
-        gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
+        GstEvent *event = gst_event_new_eos ();
+        if (dvdemux->segment_seqnum)
+          gst_event_set_seqnum (event, dvdemux->segment_seqnum);
+        gst_dvdemux_push_event (dvdemux, event);
       }
     } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_EOS) {
+      GstEvent *event = gst_event_new_eos ();
       /* for fatal errors or not-linked we post an error message */
       GST_ELEMENT_ERROR (dvdemux, STREAM, FAILED,
           (NULL), ("streaming stopped, reason %s", gst_flow_get_name (ret)));
-      gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
+      if (dvdemux->segment_seqnum)
+        gst_event_set_seqnum (event, dvdemux->segment_seqnum);
+      gst_dvdemux_push_event (dvdemux, event);
     }
     goto done;
   }
