@@ -240,10 +240,8 @@ gst_gdk_pixbuf_overlay_class_init (GstGdkPixbufOverlayClass * klass)
           GST_TYPE_GDK_PIXBUF_POSITIONING_MODE, DEFAULT_POSITIONING_MODE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_static_pad_template (element_class, &sink_template);
+  gst_element_class_add_static_pad_template (element_class, &src_template);
 
   gst_element_class_set_static_metadata (element_class,
       "GdkPixbuf Overlay", "Filter/Effect/Video",
@@ -462,6 +460,9 @@ gst_gdk_pixbuf_overlay_set_pixbuf (GstGdkPixbufOverlay * overlay,
     }
   }
 
+  if (overlay->pixels)
+    gst_buffer_unref (overlay->pixels);
+
   /* assume we have row padding even for the last row */
   /* transfer ownership of pixbuf to the buffer */
   overlay->pixels = gst_buffer_new_wrapped_full (GST_MEMORY_FLAG_READONLY,
