@@ -318,8 +318,7 @@ gst_multi_file_sink_class_init (GstMultiFileSinkClass * klass)
   GST_DEBUG_CATEGORY_INIT (gst_multi_file_sink_debug, "multifilesink", 0,
       "multifilesink element");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sinktemplate));
+  gst_element_class_add_static_pad_template (gstelement_class, &sinktemplate);
   gst_element_class_set_static_metadata (gstelement_class, "Multi-File Sink",
       "Sink/File",
       "Write buffers to a sequentially named set of files",
@@ -627,8 +626,10 @@ gst_multi_file_sink_write_buffer (GstMultiFileSink * multifilesink,
           "Writing buffer data (%" G_GSIZE_FORMAT " bytes) to new file",
           map.size);
       ret = fwrite (map.data, map.size, 1, multifilesink->file);
-      if (ret != 1)
+      if (ret != 1) {
+        gst_multi_file_sink_close_file (multifilesink, NULL);
         goto stdio_write_error;
+      }
 
       gst_multi_file_sink_close_file (multifilesink, buffer);
       break;
