@@ -20,7 +20,6 @@
 
 /**
  * SECTION:element-rtph261depay
- * @title: rtph261depay
  * @see_also: rtph261pay
  *
  * Extract encoded H.261 video frames from RTP packets according to RFC 4587.
@@ -30,12 +29,13 @@
  * aggregates the extracted stream until a complete frame is received before
  * it pushes it downstream.
  *
- * ## Example pipeline
+ * <refsect2>
+ * <title>Example pipeline</title>
  * |[
  * gst-launch-1.0 udpsrc caps='application/x-rtp, payload=31' ! rtph261depay ! avdec_h261 ! autovideosink
  * ]| This example pipeline will depayload and decode an RTP H.261 video stream.
  * Refer to the rtph261pay example to create the RTP stream.
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -105,9 +105,9 @@ gst_rtp_h261_depay_process (GstRTPBaseDepayload * depayload, GstRTPBuffer * rtp)
 
   marker = gst_rtp_buffer_get_marker (rtp);
 
-  if (payload_len < header_len + 1) {
-    /* Must have at least one byte payload */
-    GST_WARNING_OBJECT (depay, "Dropping packet with invalid payload length");
+  if (payload_len < 4) {
+    GST_WARNING_OBJECT (depay,
+        "Dropping packet with payload length invalid length");
     return NULL;
   }
 
@@ -170,7 +170,8 @@ skip:
 
       avail = gst_adapter_available (depay->adapter);
       outbuf = gst_adapter_take_buffer (depay->adapter, avail);
-      gst_rtp_drop_non_video_meta (depay, outbuf);
+      gst_rtp_drop_meta (GST_ELEMENT_CAST (depay), outbuf,
+          g_quark_from_static_string (GST_META_TAG_VIDEO_STR));
 
       /* Note that the I flag does not mean intra frame, but that the entire
        * stream is intra coded. */
