@@ -73,6 +73,7 @@ struct _GstRtpBin {
   gint            max_rtcp_rtp_time_diff;
   guint32         max_dropout_time;
   guint32         max_misorder_time;
+  gboolean        rfc7273_sync;
 
   /* a list of session */
   GSList         *sessions;
@@ -100,6 +101,7 @@ struct _GstRtpBinClass {
   /* action signals */
   void        (*clear_pt_map)         (GstRtpBin *rtpbin);
   void        (*reset_sync)           (GstRtpBin *rtpbin);
+  GstElement* (*get_session)          (GstRtpBin *rtpbin, guint session);
   RTPSession* (*get_internal_session) (GstRtpBin *rtpbin, guint session);
 
   /* session manager signals */
