@@ -350,8 +350,7 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
           "Port to bind the socket to", 0, G_MAXUINT16,
           DEFAULT_BIND_PORT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_static_pad_template (gstelement_class, &sink_template);
 
   gst_element_class_set_static_metadata (gstelement_class, "UDP packet sender",
       "Sink/Network",
@@ -1312,7 +1311,7 @@ gst_multiudpsink_start (GstBaseSink * bsink)
 
   if (sink->socket_v6) {
     GST_DEBUG_OBJECT (sink, "using configured IPv6 socket");
-    g_return_val_if_fail (g_socket_get_family (sink->socket) !=
+    g_return_val_if_fail (!sink->socket || g_socket_get_family (sink->socket) !=
         G_SOCKET_FAMILY_IPV6, FALSE);
 
     if (sink->used_socket_v6 && sink->used_socket_v6 != sink->socket_v6) {
