@@ -29,7 +29,7 @@
 /* Common device iterator */
 /**************************/
 
-#ifdef HAVE_GUDEV
+#if HAVE_GUDEV
 #include <gudev/gudev.h>
 
 struct _GstV4l2GUdevIterator
@@ -46,7 +46,7 @@ gst_v4l2_iterator_new (void)
   static const gchar *subsystems[] = { "video4linux", NULL };
   struct _GstV4l2GUdevIterator *it;
 
-  it = g_new0 (struct _GstV4l2GUdevIterator, 1);
+  it = g_slice_new0 (struct _GstV4l2GUdevIterator);
 
   it->client = g_udev_client_new (subsystems);
   it->devices = g_udev_client_query_by_subsystem (it->client, "video4linux");
@@ -92,7 +92,7 @@ gst_v4l2_iterator_free (GstV4l2Iterator * _it)
   struct _GstV4l2GUdevIterator *it = (struct _GstV4l2GUdevIterator *) _it;
   g_list_free_full (it->devices, g_object_unref);
   gst_object_unref (it->client);
-  g_free (it);
+  g_slice_free (struct _GstV4l2GUdevIterator, it);
 }
 
 #else /* No GUDEV */
@@ -110,7 +110,7 @@ gst_v4l2_iterator_new (void)
 {
   struct _GstV4l2FsIterator *it;
 
-  it = g_new0 (struct _GstV4l2FsIterator, 1);
+  it = g_slice_new0 (struct _GstV4l2FsIterator);
   it->base_idx = 0;
   it->video_idx = -1;
   it->device = NULL;
@@ -131,7 +131,7 @@ gst_v4l2_iterator_next (GstV4l2Iterator * _it)
   while (device == NULL) {
     it->video_idx++;
 
-    if (it->video_idx >= 256) {
+    if (it->video_idx >= 64) {
       it->video_idx = 0;
       it->base_idx++;
     }
@@ -160,7 +160,7 @@ gst_v4l2_iterator_free (GstV4l2Iterator * _it)
 {
   struct _GstV4l2FsIterator *it = (struct _GstV4l2FsIterator *) _it;
   g_free ((gchar *) it->parent.device_path);
-  g_free (it);
+  g_slice_free (struct _GstV4l2FsIterator, it);
 }
 
 #endif
@@ -191,10 +191,9 @@ gst_v4l2_error (gpointer element, GstV4l2Error * v4l2err)
   if (v4l2err->dbg_message)
     GST_WARNING_OBJECT (element, "error: %s", v4l2err->dbg_message);
 
-  if (GST_IS_ELEMENT (element))
-    gst_element_message_full (GST_ELEMENT (element), GST_MESSAGE_ERROR,
-        error->domain, error->code, error->message, v4l2err->dbg_message,
-        v4l2err->file, v4l2err->func, v4l2err->line);
+  gst_element_message_full (GST_ELEMENT (element), GST_MESSAGE_ERROR,
+      error->domain, error->code, error->message, v4l2err->dbg_message,
+      v4l2err->file, v4l2err->func, v4l2err->line);
 
   error->message = NULL;
   v4l2err->dbg_message = NULL;
