@@ -15,8 +15,8 @@
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
-* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-* Boston, MA 02111-1307, USA.
+* Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+* Boston, MA 02110-1301, USA.
 */
 
 /**
@@ -130,8 +130,8 @@ gst_waveform_sink_class_init (GstWaveFormSinkClass * klass)
       "Output to a sound card via WaveForm API",
       "Sebastien Moutte <sebastien@moutte.net>");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&waveformsink_sink_factory));
+  gst_element_class_add_static_pad_template (element_class,
+      &waveformsink_sink_factory);
 }
 
 static void
